package org.totschnig.myexpenses.ui

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.LinearLayout
import org.totschnig.myexpenses.R
import org.totschnig.myexpenses.databinding.BudgetSummaryBinding
import org.totschnig.myexpenses.databinding.BudgetTotalTableBinding
import org.totschnig.myexpenses.model.CurrencyUnit
import org.totschnig.myexpenses.model.Money
import org.totschnig.myexpenses.util.ColorUtils.getComplementColor
import org.totschnig.myexpenses.util.ICurrencyFormatter
import org.totschnig.myexpenses.util.ui.UiUtils
import org.totschnig.myexpenses.util.formatMoney
import org.totschnig.myexpenses.util.getBackgroundForAvailable
import org.totschnig.myexpenses.viewmodel.data.Budget
import kotlin.math.roundToInt

class BudgetSummary @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) : LinearLayout(context, attrs, defStyleAttr) {

    private val binding = BudgetSummaryBinding.inflate(LayoutInflater.from(context), this)
    private val tableBinding = BudgetTotalTableBinding.bind(binding.root)

    fun bind(
        budget: Budget,
        currencyUnit: CurrencyUnit,
        spent: Long,
        allocated: Long,
        currencyFormatter: ICurrencyFormatter
    ) {
        binding.budgetProgressTotal.finishedStrokeColor = budget.color
        binding.budgetProgressTotal.unfinishedStrokeColor = getComplementColor(budget.color)
        tableBinding.totalBudget.text = currencyFormatter.formatMoney(Money(currencyUnit, allocated))
        tableBinding.totalAmount.text = currencyFormatter.formatMoney(Money(currencyUnit, -spent))
        val available = allocated - spent
        tableBinding.totalAvailable.text = currencyFormatter.formatMoney(Money(currencyUnit, available))
        val onBudget = available >= 0
        tableBinding.totalAvailable.setBackgroundResource(getBackgroundForAvailable(onBudget))
        tableBinding.totalAvailable.setTextColor(context.resources.getColor(if (onBudget) R.color.colorIncome else R.color.colorExpense))
        val progress = if (allocated == 0L) 100 else (spent * 100f / allocated).roundToInt()
        UiUtils.configureProgress(binding.budgetProgressTotal, progress)
    }
}